@@ -96,8 +96,10 @@ static	void	hostnames	(struct parse *, FILE *);
 static	void	setdebug	(struct parse *, FILE *);
 static	void	quit		(struct parse *, FILE *);
 static	void	version		(struct parse *, FILE *);
-static	void	warning		(const char *, const char *, const char *);
-static	void	error		(const char *, const char *, const char *);
+static	void	warning		(const char *, ...)
+    __attribute__((__format__(__printf__, 1, 2)));
+static	void	error		(const char *, ...)
+    __attribute__((__format__(__printf__, 1, 2)));
 static	u_long	getkeyid	(const char *);
 
 
@@ -239,7 +241,6 @@ static	FILE *current_output;
 extern struct xcmd opcmds[];
 
 char *progname;
-volatile int debug;
 
 #ifdef NO_MAIN_ALLOWED
 CALL(ntpdc,"ntpdc",ntpdcmain);
@@ -284,6 +285,7 @@ ntpdcmain(
 
 	init_lib();	/* sets up ipv4_works, ipv6_works */
 	ssl_applink();
+	init_auth();
 
 	/* Check to see if we have IPv6. Otherwise default to IPv4 */
 	if (!ipv6_works)
@@ -313,7 +315,7 @@ ntpdcmain(
 		}
 	}
 
-	debug = DESC(DEBUG_LEVEL).optOccCt;
+	debug = OPT_VALUE_SET_DEBUG_LEVEL;
 
 	if (HAVE_OPT(INTERACTIVE)) {
 		interactive = 1;
@@ -423,7 +425,7 @@ openhost(
 	 * will return an "IPv4-mapped IPv6 address" address if you
 	 * give it an IPv4 address to lookup.
 	 */
-	strncpy(service, "ntp", sizeof(service));
+	strlcpy(service, "ntp", sizeof(service));
 	ZERO(hints);
 	hints.ai_family = ai_fam_templ;
 	hints.ai_protocol = IPPROTO_UDP;
@@ -464,10 +466,9 @@ openhost(
 	memcpy(&addr, ai->ai_addr, octets);
 
 	if (ai->ai_canonname == NULL)
-		strncpy(temphost, stoa(&addr), sizeof(temphost));
+		strlcpy(temphost, stoa(&addr), sizeof(temphost));
 	else
-		strncpy(temphost, ai->ai_canonname, sizeof(temphost));
-	temphost[sizeof(temphost) - 1] = '\0';
+		strlcpy(temphost, ai->ai_canonname, sizeof(temphost));
 
 	if (debug > 2)
 		printf("Opening host %s\n", temphost);
@@ -478,7 +479,7 @@ openhost(
 		closesocket(sockfd);
 		havehost = 0;
 	}
-	strncpy(currenthost, temphost, sizeof(currenthost));
+	strlcpy(currenthost, temphost, sizeof(currenthost));
 	
 	/* port maps to the same in both families */
 	s_port = NSRCPORT(&addr);; 
@@ -507,7 +508,7 @@ openhost(
 
 	sockfd = socket(ai->ai_family, SOCK_DGRAM, 0);
 	if (sockfd == INVALID_SOCKET) {
-		error("socket", "", "");
+		error("socket");
 		exit(-1);
 	}
 	
@@ -518,7 +519,7 @@ openhost(
 
 		if (setsockopt(sockfd, SOL_SOCKET, SO_RCVBUF,
 			       &rbufsize, sizeof(int)) == -1)
-		    error("setsockopt", "", "");
+		    error("setsockopt");
 	}
 # endif
 #endif
@@ -529,7 +530,7 @@ openhost(
 #else
 	if (connect(sockfd, ai->ai_addr, ai->ai_addrlen) == -1) {
 #endif /* SYS_VXWORKS */
-		error("connect", "", "");
+		error("connect");
 		exit(-1);
 	}
 
@@ -552,7 +553,7 @@ sendpkt(
 	)
 {
 	if (send(sockfd, xdata, xdatalen, 0) == -1) {
-		warning("write to %s failed", currenthost, "");
+		warning("write to %s failed", currenthost);
 		return -1;
 	}
 
@@ -601,7 +602,7 @@ getresponse(
 	int numrecv;
 	int seq;
 	fd_set fds;
-	int n;
+	ssize_t n;
 	int pad;
 
 	/*
@@ -630,7 +631,7 @@ getresponse(
 	n = select(sockfd+1, &fds, (fd_set *)0, (fd_set *)0, &tvo);
 
 	if (n == -1) {
-		warning("select fails", "", "");
+		warning("select fails");
 		return -1;
 	}
 	if (n == 0) {
@@ -649,7 +650,7 @@ getresponse(
 				printf("Received sequence numbers");
 				for (n = 0; n <= MAXSEQ; n++)
 				    if (haveseq[n])
-					printf(" %d,", n);
+					printf(" %zd,", n);
 				if (lastseq != 999)
 				    printf(" last frame received\n");
 				else
@@ -661,7 +662,7 @@ getresponse(
 
 	n = recv(sockfd, (char *)&rpkt, sizeof(rpkt), 0);
 	if (n == -1) {
-		warning("read", "", "");
+		warning("read");
 		return -1;
 	}
 
@@ -669,9 +670,9 @@ getresponse(
 	/*
 	 * Check for format errors.  Bug proofing.
 	 */
-	if (n < RESP_HEADER_SIZE) {
+	if (n < (ssize_t)RESP_HEADER_SIZE) {
 		if (debug)
-			printf("Short (%d byte) packet received\n", n);
+			printf("Short (%zd byte) packet received\n", n);
 		goto again;
 	}
 	if (INFO_VERSION(rpkt.rm_vn_mode) > NTP_VERSION ||
@@ -739,8 +740,8 @@ getresponse(
 	if ((size_t)datasize > (n-RESP_HEADER_SIZE)) {
 		if (debug)
 		    printf(
-			    "Received items %d, size %d (total %d), data in packet is %lu\n",
-			    items, size, datasize, (u_long)(n-RESP_HEADER_SIZE));
+			    "Received items %d, size %d (total %d), data in packet is %zu\n",
+			    items, size, datasize, n-RESP_HEADER_SIZE);
 		goto again;
 	}
 
@@ -921,11 +922,11 @@ sendrequest(
 	if (!maclen) {  
 		fprintf(stderr, "Key not found\n");
 		return 1;
-	} else if (maclen != (info_auth_hashlen + sizeof(keyid_t))) {
+	} else if (maclen != (int)(info_auth_hashlen + sizeof(keyid_t))) {
 		fprintf(stderr,
-			"%d octet MAC, %lu expected with %lu octet digest\n",
-			maclen, (u_long)(info_auth_hashlen + sizeof(keyid_t)),
-			(u_long)info_auth_hashlen);
+			"%d octet MAC, %zu expected with %zu octet digest\n",
+			maclen, (info_auth_hashlen + sizeof(keyid_t)),
+			info_auth_hashlen);
 		return 1;
 	}
 	return sendpkt(&qpkt, reqsize + maclen);
@@ -974,7 +975,7 @@ again:
 		res = select(sockfd+1, &fds, (fd_set *)0, (fd_set *)0, &tvzero);
 
 		if (res == -1) {
-			warning("polling select", "", "");
+			warning("polling select");
 			return -1;
 		} else if (res > 0)
 
@@ -1030,7 +1031,7 @@ again:
 			if (implcode == IMPL_XNTPD)
 				break;
 			(void) fprintf(stderr,
-				       "***Server implementation incompatable with our own\n");
+				       "***Server implementation incompatible with our own\n");
 			break;
 		case INFO_ERR_REQ:
 			(void) fprintf(stderr,
@@ -1124,6 +1125,11 @@ docmd(
 	/*
 	 * Tokenize the command line.  If nothing on it, return.
 	 */
+	if (strlen(cmdline) >= MAXLINE) {
+		fprintf(stderr, "***Command ignored, more than %d characters:\n%s\n",
+			MAXLINE - 1, cmdline);
+		return;
+	}
 	tokenize(cmdline, tokens, &ntok);
 	if (ntok == 0)
 	    return;
@@ -1419,10 +1425,8 @@ getnetnum(
 	int af
 	)
 {
-	int sockaddr_len;
 	struct addrinfo hints, *ai = NULL;
 
-	sockaddr_len = SIZEOF_SOCKADDR(af);
 	ZERO(hints);
 	hints.ai_flags = AI_CANONNAME;
 #ifdef AI_ADDRCONFIG
@@ -1439,11 +1443,11 @@ getnetnum(
 				    LENHOSTNAME, NULL, 0, 0);
 		return 1;
 	} else if (getaddrinfo(hname, "ntp", &hints, &ai) == 0) {
-		NTP_INSIST(sizeof(*num) >= ai->ai_addrlen);
+		INSIST(sizeof(*num) >= ai->ai_addrlen);
 		memcpy(num, ai->ai_addr, ai->ai_addrlen);
 		if (fullhost != NULL) {
 			if (ai->ai_canonname != NULL)
-				strncpy(fullhost, ai->ai_canonname,
+				strlcpy(fullhost, ai->ai_canonname,
 					LENHOSTNAME);
 			else
 				getnameinfo(&num->sa, SOCKLEN(num),
@@ -1451,11 +1455,10 @@ getnetnum(
 					    0, 0);
 		}
 		return 1;
-	} else {
-		(void) fprintf(stderr, "***Can't find host %s\n", hname);
-		return 0;
 	}
-	/*NOTREACHED*/
+	fprintf(stderr, "***Can't find host %s\n", hname);
+
+	return 0;
 }
 
 
@@ -1867,34 +1870,44 @@ version(
 }
 
 
+static void __attribute__((__format__(__printf__, 1, 0)))
+vwarning(const char *fmt, va_list ap)
+{
+	int serrno = errno;
+	(void) fprintf(stderr, "%s: ", progname);
+	vfprintf(stderr, fmt, ap);
+	(void) fprintf(stderr, ": %s\n", strerror(serrno));
+}
+
 /*
  * warning - print a warning message
  */
-static void
+static void __attribute__((__format__(__printf__, 1, 2)))
 warning(
 	const char *fmt,
-	const char *st1,
-	const char *st2
+	...
 	)
 {
-	(void) fprintf(stderr, "%s: ", progname);
-	(void) fprintf(stderr, fmt, st1, st2);
-	(void) fprintf(stderr, ": ");
-	perror("");
+	va_list ap;
+	va_start(ap, fmt);
+	vwarning(fmt, ap);
+	va_end(ap);
 }
 
 
 /*
  * error - print a message and exit
  */
-static void
+static void __attribute__((__format__(__printf__, 1, 2)))
 error(
 	const char *fmt,
-	const char *st1,
-	const char *st2
+	...
 	)
 {
-	warning(fmt, st1, st2);
+	va_list ap;
+	va_start(ap, fmt);
+	vwarning(fmt, ap);
+	va_end(ap);
 	exit(1);
 }
 
