@@ -355,8 +355,8 @@ main(
 	fstamp = (u_int)(epoch + JAN_1970);
 
 	optct = ntpOptionProcess(&ntp_keygenOptions, argc, argv);
-	argc -= optct;
-	argv += optct;
+	argc -= optct;	// Just in case we care later.
+	argv += optct;	// Just in case we care later.
 
 #ifdef OPENSSL
 	if (SSLeay() == SSLEAY_VERSION_NUMBER)
